# TTTLetter class
"""Implements the functionality of a letter in Boggle."""

from graphics import *
from board import Board

class TTTLetter:
    """A TTT letter has several attributes that define it:
           *  _row, _col coordinates indicate its position in the grid (ints)
           *  _textObj denotes the Text object from the graphics module,
              which has attributes such as size, style, color, etc
              and supports methods such as getText(), setText() etc.
           *  _rect denotes the Rectangle object from the graphics module,
              which has attributes such as color and supports methods such as 
              getFillColor(), setFillColor() etc.
    """

    __slots__ = ['_row', '_col', '_textObj', '_rect']

    def __init__(self, board, col=-1, row=-1, letter=""):

        # variables needed for graphical testing
        xInset = board.getXInset()
        yInset = board.getYInset()
        size = board.getSize()
        win = board.getWin()

        # set row and column attributes
        self._col = col
        self._row = row

        # make rectangle and add to graphical window
        p1 = Point(xInset + size * col, yInset + size * row)
        p2 = Point(xInset + size * (col + 1), yInset + size * (row + 1))        
        self._rect = board._makeRect(p1, p2, "white")

        # update text in center of rectangle
        self._textObj = Text(self._rect.getCenter(), letter)
        self._textObj.draw(win)

    def getLetter(self):
        """Returns letter (text of type str) associated with self._textObj"""
        return self._textObj.getText()

    def setLetter(self, char):
        self._textObj.setText(char)
        if char == 'X':
            self._rect.setFillColor("light blue")
        elif char == 'O':
            self._rect.setFillColor("pink")
        else:
            self._rect.setFillColor("white")

    def __str__(self):
        l, col, row = self.getLetter(), self._col, self._row
        return "{} at Board position ({}, {})".format(l, col, row)

    def __repr__(self):
        return str(self)

if __name__ == "__main__":
    win = GraphWin("Tic Tac Toe", 400, 400)
    board = Board(win, rows=3, cols=3)

    letter = TTTLetter(board, 1, 1, "A")
    letter2 = TTTLetter(board, 1, 2, "O")
    letter3 = TTTLetter(board, 2, 1, "B")

    letter2.setLetter("O")
    print(letter2)

    # pause and wait for mouse click
    point = win.getMouse()
